'''
Created on Oct 2, 2013

@author: "Duarte Meneses <duarte.meneses@cern.ch>"
'''
import gfal2
import sys
import base
import os
from base import CommandBase
import errno
from progress import Progress

class GfalCommands(CommandBase):
    @base.arg('-m', '--mode', action="store", type=int, default=755, help="display hidden files")
    @base.arg('-p', '--parents', action='store_true', help='no error if existing, make parent directories as needed')
    @base.arg('directory', action='store', nargs='+', type=str, help="Directory's uri.")
    def execute_mkdir(self):
        '''Makes directories. By default, it sets file mode 0755.'''    
        if self.params.mode:
            try:
                mode = int(str(self.params.mode), 8)
            except ValueError:
                mode = 0755
        else:
            mode = 0755
        for d in self.params.directory:
            if self.params.parents:
                self.context.mkdir_rec(d, mode)
            else:
                self.context.mkdir(d, mode)
    
    @base.arg("file", action='store', type=str, help='uri of the file to be written')
    def execute_save(self):
        '''Reads from stdin and writes to a file. If the file exists, it will be overwritten'''
        f = self.context.open(self.params.file, 'w')
        
        while True:
            b = sys.stdin.read(65000)
            if b:
                f.write(b)
            else:
                break
    
    @base.arg("file", action='store', nargs='+', type=str, help='uri of the file to be displayed')
    def execute_cat(self):
        '''Sends to stdout the contents of files'''
        
        for fname in self.params.file:
            f = self.context.open(fname, 'r')
    
            while True:
                b = f.read(65000)
                sys.stdout.write(b)
                if not b:
                    break
    
    @base.arg('-f',"--force", action='store_true', help="if destination file(s) cannot be overwritten, delete it and try again")
    @base.arg('-n',"--nbstreams", type=int, default=None, help="specify the maximum number of parallel streams to use for the copy")
    @base.arg('-s',"--src-spacetoken", type=str, default="", help="source spacetoken to use for the transfer")
    @base.arg('-S',"--dst-spacetoken", type=str, default="", help="destination spacetoken to use for the transfer")
    @base.arg('-T',"--transfer-timeout", type=int, default=None, help="global timeout for the transfer operation")
    @base.arg('src', type=str, help="source file")
    @base.arg('dst', type=str, help="destination file")
    def execute_copy(self):
        '''Copy a file'''
        
        try:
            file_size = self.context.stat(self.params.src).st_size
        except Exception:
            file_size = None
        
        if not self.params.verbose and sys.stdout.isatty():
            self.progress_bar = Progress('Copying')
            self.progress_bar.update(total_size=file_size)
        
        def event_callback(event):
            if self.params.verbose:
                print "event: %s" % str(event)
                
        def monitor_callback(src, dst, avg, inst, trans, elapsed):
            if self.params.verbose:
                print "monitor: %s %s %s %s %s %s" % (str(src), str(dst), \
                            str(avg), str(inst), str(trans), str(elapsed))
                
            if self.progress_bar:
                self.progress_bar.update(trans, file_size, avg, elapsed)
        
        #set up transfer parameters. Leave defaults unless specified
        t = self.context.transfer_parameters()
        t.event_callback = event_callback
        t.monitor_callback = monitor_callback
        if self.params.nbstreams:
            t.nbstreams = self.params.nbstreams
        if self.params.transfer_timeout:
            t.transfer_timeout = self.params.timeout
        if self.params.src_spacetoken:
            t.src_spacetoken = self.params.src_spacetoken
        if self.params.dst_spacetoken:
            t.dst_spacetoken = self.params.dst_spacetoken
        
        
        if self.params.dst[-1] == '/':
            self.params.dst += os.path.basename(self.params.src)
        
        ret = -1
        if self.progress_bar:
            self.progress_bar.start()
        try:
            ret = self.context.filecopy(t, self.params.src, self.params.dst)
        finally: 
            if self.progress_bar:
                self.progress_bar.stop(ret == 0)


    @base.arg('file', action='store', type=str, help="file uri")
    @base.arg('attribute', nargs='?', type=str, help="attribute to retrieve or set. To set, use key=value")
    def execute_xattr(self):
        if self.params.attribute is not None:
            if '=' in self.params.attribute:
                i = self.params.attribute.find("=")
                attr  = self.params.attribute[:i]
                v = self.params.attribute[(i+1):]
                
                if len(attr) > 0 and len(v) > 0:
                    self.context.setxattr(self.params.file, attr, v, 0)    
            else:
                v = self.context.getxattr(self.params.file, self.params.attribute)
                sys.stdout.write(v + '\n')
        else:
            l = self.context.listxattr(self.params.file)
            
            for attr in l:
                v = self.context.getxattr(self.params.file, attr)
                sys.stdout.write(attr + ' = ' + v + '\n')

    @base.arg('file', action='store', type=str, help="file uri to use for checksum calculation")
    @base.arg('checksum_type', action='store', type=str, help="checksum algorithm to use. For example: ADLER32, CRC32, MD5.")
    def execute_sum(self):
        '''Calculates the checksum of a file'''
        checksum = self.context.checksum(self.params.file, self.params.checksum_type)
        sys.stdout.write(self.params.file + ' ' + checksum + '\n')
        
    @base.arg("-r", "-R", "--recursive", action='store_true', help="remove directories and their contents recursively")
    @base.arg("file", action='store', nargs='+', type=str, help="uri(s) of the file(s) to be deleted")
    def execute_rm(self):
        '''Removes files or directories'''
        for f in self.params.file:
            try:
                self.context.unlink(f)
            except gfal2.GError as e:
                if (e.code == errno.EISDIR or e.code == errno.ENOTEMPTY) and self.params.recursive:
                    self.__rm_dir(f)
                    self.context.rmdir(f)
                else:
                    raise
        
    def __rm_dir(self, directory):
        exclude = ['.', '..']
        contents = self.context.listdir(directory)
        if directory[-1] != '/':
            directory = directory + '/'
        
        for c in contents:
            if c in exclude:
                continue
            try:
                self.context.unlink(directory + c)
            except gfal2.GError as e:
                if e.code == errno.EISDIR or e.code == errno.ENOTEMPTY:
                    self.__rm_dir(directory + c)
                    self.context.rmdir(directory + c)
                else:
                    raise
    
    def __filter_hidden(self, f_list):
        def filter_f_list(mfile):
            return mfile[0] != '.'
        return filter(filter_f_list, f_list)

